# gemato: OpenPGP signature support tests
# vim:fileencoding=utf-8
# (c) 2017-2018 Michał Górny
# Licensed under the terms of 2-clause BSD license

import base64
import datetime
import io
import os.path
import shutil
import tempfile
import unittest

import gemato.cli
import gemato.compression
import gemato.manifest
import gemato.openpgp
import gemato.recursiveloader


PUBLIC_KEY = b'''
-----BEGIN PGP PUBLIC KEY BLOCK-----

mQENBFnwXJMBCACgaTVz+d10TGL9zR920sb0GBFsitAJ5ZFzO4E0cg3SHhwI+reM
JQ6LLKmHowY/E1dl5FBbnJoRMxXP7/eScQ7HlhYj1gMPN5XiS2pkPwVkmJKBDV42
DLwoytC+ot0frRTJvSdEPCX81BNMgFiBSpkeZfXqb9XmU03bh6mFnrdd4CsHpTQG
csVXHK8QKhaxuqmHTALdpSzKCb/r0N/Z3sQExZhfLcBf/9UUVXj44Nwc6ooqZLRi
zHydxwQdxNu0aOFGEBn9WTi8Slf7MfR/pF0dI8rs9w6zMzVEq0lhDPpKFGDveoGf
g/+TpvBNXZ7DWH23GM4kID3pk4LLMc24U1PhABEBAAG0D2dlbWF0byB0ZXN0IGtl
eYkBRgQTAQoAMBYhBIHhLBa9jc1gvhgIRRNogOcqexOEBQJZ8FyTAhsDBQsJCg0E
AxUKCAIeAQIXgAAKCRATaIDnKnsThCnkB/0fhTH230idhlfZhFbVgTLxrj4rpsGg
20K8HkMaWzChsONdKkqYaYuRcm2UQZ0Kg5rm9jQsGYuAnzH/7XwmOleY95ycVfBk
je9aXF6BEoGick6C/AK5w77vd1kcBtJDrT4I7vwD4wRkyUdCkpVMVT4z4aZ7lHJ4
ECrrrI/mg0b+sGRyHfXPvIPp7F2959L/dpbhBZDfMOFC0A9LBQBJldKFbQLg3xzX
4tniz/BBrp7KjTOMKU0sufsedI50xc6cvCYCwJElqo86vv69klZHahE/k9nJaUAM
jCvJNJ7pU8YnJSRTQDH0PZEupAdzDU/AhGSrBz5+Jr7N0pQIxq4duE/Q
=r7JK
-----END PGP PUBLIC KEY BLOCK-----
'''

EXPIRED_PUBLIC_KEY = b'''
-----BEGIN PGP PUBLIC KEY BLOCK-----

mQENBFnwXJMBCACgaTVz+d10TGL9zR920sb0GBFsitAJ5ZFzO4E0cg3SHhwI+reM
JQ6LLKmHowY/E1dl5FBbnJoRMxXP7/eScQ7HlhYj1gMPN5XiS2pkPwVkmJKBDV42
DLwoytC+ot0frRTJvSdEPCX81BNMgFiBSpkeZfXqb9XmU03bh6mFnrdd4CsHpTQG
csVXHK8QKhaxuqmHTALdpSzKCb/r0N/Z3sQExZhfLcBf/9UUVXj44Nwc6ooqZLRi
zHydxwQdxNu0aOFGEBn9WTi8Slf7MfR/pF0dI8rs9w6zMzVEq0lhDPpKFGDveoGf
g/+TpvBNXZ7DWH23GM4kID3pk4LLMc24U1PhABEBAAG0D2dlbWF0byB0ZXN0IGtl
eYkBTAQTAQoANgIbAwULCQoNBAMVCggCHgECF4AWIQSB4SwWvY3NYL4YCEUTaIDn
KnsThAUCWfEJZAUJAAH+UQAKCRATaIDnKnsThJTJB/9nXG1vgEBXHp8JsgkbmsAA
WzcSsdmuRFcr2FI3KDYJ0G7rmBpirJuAaGbWS/2+3BmQGVlOf77RjeC6CtI/DH4U
Tw3hcI7FYJrRdILV+p3HTkLhPs5fNjxH8bTyKthEE8pM0gQ3fuZxsaNnv1XbSpf0
P+d/y06ehvGCVYyEe4MHPV6f6YgCrP+ePqQvMqEpvlSizZE/HoFoKy7Ik4u2fDOH
5RRmIoNLv8j1gOKwp5+SncsuXVdQucY7jdFWSgECOAGIRvzBbGDq9+6ccCQHiOOz
ncaJWqeCHuTvNj9WfoyvKXM+hpQUdSaTURgz4a92htIGpON5wN7o32VuJz2nWXS4
=RwD3
-----END PGP PUBLIC KEY BLOCK-----
'''

REVOKED_PUBLIC_KEY = b'''
-----BEGIN PGP PUBLIC KEY BLOCK-----

mQENBFnwXJMBCACgaTVz+d10TGL9zR920sb0GBFsitAJ5ZFzO4E0cg3SHhwI+reM
JQ6LLKmHowY/E1dl5FBbnJoRMxXP7/eScQ7HlhYj1gMPN5XiS2pkPwVkmJKBDV42
DLwoytC+ot0frRTJvSdEPCX81BNMgFiBSpkeZfXqb9XmU03bh6mFnrdd4CsHpTQG
csVXHK8QKhaxuqmHTALdpSzKCb/r0N/Z3sQExZhfLcBf/9UUVXj44Nwc6ooqZLRi
zHydxwQdxNu0aOFGEBn9WTi8Slf7MfR/pF0dI8rs9w6zMzVEq0lhDPpKFGDveoGf
g/+TpvBNXZ7DWH23GM4kID3pk4LLMc24U1PhABEBAAGJATYEIAEIACAWIQSB4SwW
vY3NYL4YCEUTaIDnKnsThAUCWl4LpQIdAAAKCRATaIDnKnsThCvQB/9gzrXiRv7g
7UIzwRvTBMVXbKlbwjJpI0XnjdZmlYVis5y4ZWxh65z8j94lV4NmTtDtAdlMN8Xd
OyTVWPGKN5cJMRLMzMRWb+aQV1fCEfwJgngE0hQe0w2dPwuVQQVP3Kv8CC+4f2lQ
ummcgPW2LVEN3HihAwx9VWA91JSlrsX3luNSvTi2c63BM9YqGb64nJc1sAWqxzDy
x157gzt0AHKAAQ+Hmwhqt0vnR8MyKJYo60PwNUkzWlUhOLaBpb7WvHAAmva14Rw+
fCaldU4iFlC5oJrj0jE/yKvGG6SuSYZaS9O0H/UNI5vF8Y/HgGM0i8+NJxXu0hud
NRH/MmEilKxUtA9nZW1hdG8gdGVzdCBrZXmJAUYEEwEKADAWIQSB4SwWvY3NYL4Y
CEUTaIDnKnsThAUCWfBckwIbAwULCQoNBAMVCggCHgECF4AACgkQE2iA5yp7E4Qp
5Af9H4Ux9t9InYZX2YRW1YEy8a4+K6bBoNtCvB5DGlswobDjXSpKmGmLkXJtlEGd
CoOa5vY0LBmLgJ8x/+18JjpXmPecnFXwZI3vWlxegRKBonJOgvwCucO+73dZHAbS
Q60+CO78A+MEZMlHQpKVTFU+M+Gme5RyeBAq66yP5oNG/rBkch31z7yD6exdvefS
/3aW4QWQ3zDhQtAPSwUASZXShW0C4N8c1+LZ4s/wQa6eyo0zjClNLLn7HnSOdMXO
nLwmAsCRJaqPOr7+vZJWR2oRP5PZyWlADIwryTSe6VPGJyUkU0Ax9D2RLqQHcw1P
wIRkqwc+fia+zdKUCMauHbhP0A==
=Zvmi
-----END PGP PUBLIC KEY BLOCK-----
'''

PRIVATE_KEY = b'''
-----BEGIN PGP PRIVATE KEY BLOCK-----

lQOYBFnwXJMBCACgaTVz+d10TGL9zR920sb0GBFsitAJ5ZFzO4E0cg3SHhwI+reM
JQ6LLKmHowY/E1dl5FBbnJoRMxXP7/eScQ7HlhYj1gMPN5XiS2pkPwVkmJKBDV42
DLwoytC+ot0frRTJvSdEPCX81BNMgFiBSpkeZfXqb9XmU03bh6mFnrdd4CsHpTQG
csVXHK8QKhaxuqmHTALdpSzKCb/r0N/Z3sQExZhfLcBf/9UUVXj44Nwc6ooqZLRi
zHydxwQdxNu0aOFGEBn9WTi8Slf7MfR/pF0dI8rs9w6zMzVEq0lhDPpKFGDveoGf
g/+TpvBNXZ7DWH23GM4kID3pk4LLMc24U1PhABEBAAEAB/sEgeBMIXW9ClZvvj9H
lfWcLz7yF1ZwKMC1BbOENz43LLxp7i2RJQtrErayxnxq8k6u4ML3SAe2OwK+ZIZG
2aFqL0fw+tb8KvotsSPMrE6o/HaFZMxEZYg19zj1WlsvRCxE3OlJDA2fNJBUQnj6
LQ/vYDsQOtM+VRHnfMDhLcwGObZnNPMwtmwkHLKWTgyTwAGnLObSheVutVbdyU6+
wI3UXwAoilW2e+9pKtwaODjqT7pQ2maVSCY4MPGdLQpbPy61COstdpK/hRdI3liL
uwszdlnT1QhiLsOTHPt4JjYdv2jgDjQobbe/ziKNzFp1eoMHDkbjzAh7oD2FxJcZ
EYLnBADE5oryW+9GlyYQe3x74QD5BGTZfvJctvEOgUg8BsoIfXJgBzwnEwOD0XBg
Jcl5qgt3IBH9Fn3JnYMpw12SEG2W4N8VCIBxIkDEBABVJfp1Q7HAJ8GSmzENnvt1
iaAZPUscaFVpMyuajsCDmyK92NMymGiNAb1H5MU4gaFGaEaajwQA0I7gglsehQA2
MSyJD0Uj+0b6n9KtiUzjyWEOcITXn4buf4O8Llor8gU0BWuv3hmIcvNsuJfmgXav
Vxq2UHtiGaO7T9Vk4Sr8MKS9EYrLNbK41Lyb+tjxk3jYjEyFqCDNEtWKIZR4ENdR
jo5gYKBtuqv1AYYSkflOTeaRlv/kIo8D/jVcyjmO19tNJM8lQE1xCvhp5maXOoSk
1UoUmDprsKA2Em47J83sVivrIwBySB2n9srQynnV+8I47mX7YzYtNQ6uXdL3p/5e
FRW+yfqVCShhSfyQdOmJ978UyQEwY0+0hhK372KatmaL9KEkKSuXgsqshv3XiB9y
u3Su1jw5y2IQNP20D2dlbWF0byB0ZXN0IGtleYkBRgQTAQoAMBYhBIHhLBa9jc1g
vhgIRRNogOcqexOEBQJZ8FyTAhsDBQsJCg0EAxUKCAIeAQIXgAAKCRATaIDnKnsT
hCnkB/0fhTH230idhlfZhFbVgTLxrj4rpsGg20K8HkMaWzChsONdKkqYaYuRcm2U
QZ0Kg5rm9jQsGYuAnzH/7XwmOleY95ycVfBkje9aXF6BEoGick6C/AK5w77vd1kc
BtJDrT4I7vwD4wRkyUdCkpVMVT4z4aZ7lHJ4ECrrrI/mg0b+sGRyHfXPvIPp7F29
59L/dpbhBZDfMOFC0A9LBQBJldKFbQLg3xzX4tniz/BBrp7KjTOMKU0sufsedI50
xc6cvCYCwJElqo86vv69klZHahE/k9nJaUAMjCvJNJ7pU8YnJSRTQDH0PZEupAdz
DU/AhGSrBz5+Jr7N0pQIxq4duE/Q
=wOFB
-----END PGP PRIVATE KEY BLOCK-----

'''

PRIVATE_KEY_ID = b'0x136880E72A7B1384'

MALFORMED_PUBLIC_KEY = b'''
-----BEGIN PGP PUBLIC KEY BLOCK-----

mQENBFnwXJMBCACgaTVz+d10TGL9zR920sb0GBFsitAJ5ZFzO4E0cg3SHhwI+reM
JQ6LLKmHowY/E1dl5FBbnJoRMxXP7/eScQ7HlhYj1gMPN5XiS2pkPwVkmJKBDV42
DLwoytC+ot0frRTJvSdEPCX81BNMgFiBSpkeZfXqb9XmU03bh6mFnrdd4CsHpTQG
csVXHK8QKhaxuqmHTALdpSzKCb/r0N/Z3sQExZhfLcBf/9UUVXj44Nwc6ooqZLRi
zHydxwQdxNu0aOFGEBn9WTi8Slf7MfR/pF0dI8rs9w6zMzVEq0lhDPpKFGDveoGf
g/+TpvBNXZ7DWH23GM4kID3pk4LLMc24U1PhABEBAAG0D2dlbWF0byB0ZXN0IGtl
eYkBRgQTAQoAMBYhBIHhLBa9jc1gvhgIRRNogOcqexOEBQJZ8FyTAhsDBQsJCg0E
AxUKCAIeAQIXgAAKCRATaIDnKnsThCnkB/0fhTH230idhlfZhFbVgTLxrj4rpsGg
20K8HkMaWzshsONdKkqYaYuRcm2UQZ0Kg5rm9jQsGYuAnzH/7XwmOleY95ycVfBk
je9aXF6BEoGick6C/AK5w77vd1kcBtJDrT4I7vwD4wRkyUdCkpVMVT4z4aZ7lHJ4
ECrrrI/mg0b+sGRyHfXPvIPp7F2959L/dpbhBZDfMOFC0A9LBQBJldKFbQLg3xzX
4tniz/BBrp7KjTOMKU0sufsedI50xc6cvCYCwJElqo86vv69klZHahE/k9nJaUAM
jCvJNJ7pU8YnJSRTQDH0PZEupAdzDU/AhGSrBz5+Jr7N0pQIxq4duE/Q
=r7JK
-----END PGP PUBLIC KEY BLOCK-----
'''

SIGNED_MANIFEST = u'''
-----BEGIN PGP SIGNED MESSAGE-----
Hash: SHA256

TIMESTAMP 2017-10-22T18:06:41Z
MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
IGNORE local
DATA myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-----BEGIN PGP SIGNATURE-----

iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAloCx+YACgkQE2iA5yp7
E4TYrwf+JxjkVDNtvSN3HjQmdtcayLsaliw/2kqjoaQKs0lZD8+NRe7xPmwSm4bP
XKfoouJ0+/s87vuYJpBBCjtUDA9C9yZIeRTo8+eW6XsZbRRUmUD5ylTS+FpSsUrS
bEyYk4yZQMYrat+GQ1QBv+625nqnSDv5LZHBBZ/rG36GGlwHPbIKIishnDfdG2QQ
zuxkqepNq4Inzp//ES7Bv4qbTzyBI//HzfY31vOgdhhs5N5Ytez3Xxv/KNOTYdi1
ZIfqeaQ4NoefmxQunyEjT+8X2DMaEeHQni7dwjQc+FiN4ReV9aWbLo2O2cArqEHR
mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
=khff
-----END PGP SIGNATURE-----
'''

DASH_ESCAPED_SIGNED_MANIFEST = u'''
-----BEGIN PGP SIGNED MESSAGE-----
Hash: SHA256

- TIMESTAMP 2017-10-22T18:06:41Z
- MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
IGNORE local
- DATA myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
- DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-----BEGIN PGP SIGNATURE-----

iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAloCx+YACgkQE2iA5yp7
E4TYrwf+JxjkVDNtvSN3HjQmdtcayLsaliw/2kqjoaQKs0lZD8+NRe7xPmwSm4bP
XKfoouJ0+/s87vuYJpBBCjtUDA9C9yZIeRTo8+eW6XsZbRRUmUD5ylTS+FpSsUrS
bEyYk4yZQMYrat+GQ1QBv+625nqnSDv5LZHBBZ/rG36GGlwHPbIKIishnDfdG2QQ
zuxkqepNq4Inzp//ES7Bv4qbTzyBI//HzfY31vOgdhhs5N5Ytez3Xxv/KNOTYdi1
ZIfqeaQ4NoefmxQunyEjT+8X2DMaEeHQni7dwjQc+FiN4ReV9aWbLo2O2cArqEHR
mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
=khff
-----END PGP SIGNATURE-----
'''

MODIFIED_SIGNED_MANIFEST = u'''
-----BEGIN PGP SIGNED MESSAGE-----
Hash: SHA256

TIMESTAMP 2017-10-22T18:06:41Z
MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
IGNORE local
DATA myebuild-0.ebuild 32
MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-----BEGIN PGP SIGNATURE-----

iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAloCx+YACgkQE2iA5yp7
E4TYrwf+JxjkVDNtvSN3HjQmdtcayLsaliw/2kqjoaQKs0lZD8+NRe7xPmwSm4bP
XKfoouJ0+/s87vuYJpBBCjtUDA9C9yZIeRTo8+eW6XsZbRRUmUD5ylTS+FpSsUrS
bEyYk4yZQMYrat+GQ1QBv+625nqnSDv5LZHBBZ/rG36GGlwHPbIKIishnDfdG2QQ
zuxkqepNq4Inzp//ES7Bv4qbTzyBI//HzfY31vOgdhhs5N5Ytez3Xxv/KNOTYdi1
ZIfqeaQ4NoefmxQunyEjT+8X2DMaEeHQni7dwjQc+FiN4ReV9aWbLo2O2cArqEHR
mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
=khff
-----END PGP SIGNATURE-----
'''

KEY_FINGERPRINT = '81E12C16BD8DCD60BE180845136880E72A7B1384'
SIG_TIMESTAMP = datetime.datetime(2017, 11, 8, 9, 1, 26)


def strip_openpgp(text):
    lines = text.lstrip().splitlines()
    start = lines.index('')
    stop = lines.index('-----BEGIN PGP SIGNATURE-----')
    return '\n'.join(lines[start+1:stop-start+2]) + '\n'


class SignedManifestTest(unittest.TestCase):
    """
    Test whether signed Manifest is read correctly.
    """

    def test_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            m.load(f, verify_openpgp=False)
        self.assertIsNotNone(m.find_timestamp())
        self.assertIsNotNone(m.find_path_entry('myebuild-0.ebuild'))
        self.assertFalse(m.openpgp_signed)
        self.assertIsNone(m.openpgp_signature)

    def test_dash_escaped_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(DASH_ESCAPED_SIGNED_MANIFEST) as f:
            m.load(f, verify_openpgp=False)
        self.assertIsNotNone(m.find_timestamp())
        self.assertIsNotNone(m.find_path_entry('myebuild-0.ebuild'))
        self.assertFalse(m.openpgp_signed)
        self.assertIsNone(m.openpgp_signature)

    def test_modified_manifest_load(self):
        """
        Modified Manifest should load correctly since we do not enforce
        implicit verification.
        """
        m = gemato.manifest.ManifestFile()
        with io.StringIO(MODIFIED_SIGNED_MANIFEST) as f:
            m.load(f, verify_openpgp=False)
        self.assertIsNotNone(m.find_timestamp())
        self.assertIsNotNone(m.find_path_entry('myebuild-0.ebuild'))
        self.assertFalse(m.openpgp_signed)
        self.assertIsNone(m.openpgp_signature)

    def test_junk_before_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO('IGNORE test\n' + SIGNED_MANIFEST) as f:
            self.assertRaises(gemato.exceptions.ManifestUnsignedData,
                    m.load, f, verify_openpgp=False)

    def test_junk_after_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST + 'IGNORE test\n') as f:
            self.assertRaises(gemato.exceptions.ManifestUnsignedData,
                    m.load, f, verify_openpgp=False)

    def test_signed_manifest_terminated_before_data(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO('\n'.join(SIGNED_MANIFEST.splitlines()[:3])) as f:
            self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                    m.load, f, verify_openpgp=False)

    def test_signed_manifest_terminated_before_signature(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO('\n'.join(SIGNED_MANIFEST.splitlines()[:7])) as f:
            self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                    m.load, f, verify_openpgp=False)

    def test_signed_manifest_terminated_before_end(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO('\n'.join(SIGNED_MANIFEST.splitlines()[:15])) as f:
            self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                    m.load, f, verify_openpgp=False)

    def test_recursive_manifest_loader(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(MODIFIED_SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest'),
                    verify_openpgp=False)
            self.assertFalse(m.openpgp_signed)
            self.assertIsNone(m.openpgp_signature)
        finally:
            shutil.rmtree(d)

    def test_cli(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(MODIFIED_SIGNED_MANIFEST)

            os.mkdir(os.path.join(d, 'eclass'))
            with io.open(os.path.join(d, 'eclass/Manifest'), 'w'):
                pass
            with io.open(os.path.join(d, 'myebuild-0.ebuild'), 'wb') as f:
                f.write(b'12345678901234567890123456789012')
            with io.open(os.path.join(d, 'metadata.xml'), 'w'):
                pass

            self.assertEqual(
                    gemato.cli.main(['gemato', 'verify',
                        '--no-openpgp-verify', d]),
                    0)
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_save_manifest(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(MODIFIED_SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest'),
                    verify_openpgp=False)
            self.assertFalse(m.openpgp_signed)
            self.assertIsNone(m.openpgp_signature)
            m.save_manifest('Manifest')

            with io.open(os.path.join(d, 'Manifest'), 'r') as f:
                self.assertEqual(f.read(),
                        strip_openpgp(MODIFIED_SIGNED_MANIFEST))
        finally:
            shutil.rmtree(d)


class OpenPGPCorrectKeyTest(unittest.TestCase):
    """
    Tests performed with correct OpenPGP key set.
    """

    def setUp(self):
        self.env = gemato.openpgp.OpenPGPEnvironment()
        try:
            self.env.import_key(io.BytesIO(PUBLIC_KEY))
        except gemato.exceptions.OpenPGPNoImplementation as e:
            self.env.close()
            raise unittest.SkipTest(str(e))
        except RuntimeError:
            self.env.close()
            raise unittest.SkipTest('Unable to import OpenPGP key')

    def tearDown(self):
        self.env.close()

    def test_verify_manifest(self):
        with io.StringIO(SIGNED_MANIFEST) as f:
            sig = self.env.verify_file(f)
            self.assertEqual(sig.fingerprint, KEY_FINGERPRINT)
            self.assertEqual(sig.timestamp, SIG_TIMESTAMP)
            self.assertIsNone(sig.expire_timestamp)
            self.assertEqual(sig.primary_key_fingerprint, KEY_FINGERPRINT)

    def test_verify_dash_escaped_manifest(self):
        with io.StringIO(DASH_ESCAPED_SIGNED_MANIFEST) as f:
            sig = self.env.verify_file(f)
            self.assertEqual(sig.fingerprint, KEY_FINGERPRINT)
            self.assertEqual(sig.timestamp, SIG_TIMESTAMP)
            self.assertIsNone(sig.expire_timestamp)
            self.assertEqual(sig.primary_key_fingerprint, KEY_FINGERPRINT)

    def test_verify_modified_manifest(self):
        with io.StringIO(MODIFIED_SIGNED_MANIFEST) as f:
            self.assertRaises(gemato.exceptions.OpenPGPVerificationFailure,
                    self.env.verify_file, f)

    def test_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            m.load(f, openpgp_env=self.env)
        self.assertIsNotNone(m.find_timestamp())
        self.assertIsNotNone(m.find_path_entry('myebuild-0.ebuild'))
        self.assertTrue(m.openpgp_signed)
        self.assertEqual(m.openpgp_signature.fingerprint, KEY_FINGERPRINT)
        self.assertEqual(m.openpgp_signature.timestamp, SIG_TIMESTAMP)
        self.assertIsNone(m.openpgp_signature.expire_timestamp)
        self.assertEqual(m.openpgp_signature.primary_key_fingerprint, KEY_FINGERPRINT)

    def test_dash_escaped_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(DASH_ESCAPED_SIGNED_MANIFEST) as f:
            m.load(f, openpgp_env=self.env)
        self.assertIsNotNone(m.find_timestamp())
        self.assertIsNotNone(m.find_path_entry('myebuild-0.ebuild'))
        self.assertTrue(m.openpgp_signed)
        self.assertEqual(m.openpgp_signature.fingerprint, KEY_FINGERPRINT)
        self.assertEqual(m.openpgp_signature.timestamp, SIG_TIMESTAMP)
        self.assertIsNone(m.openpgp_signature.expire_timestamp)
        self.assertEqual(m.openpgp_signature.primary_key_fingerprint, KEY_FINGERPRINT)

    def test_modified_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(MODIFIED_SIGNED_MANIFEST) as f:
            self.assertRaises(gemato.exceptions.OpenPGPVerificationFailure,
                m.load, f, openpgp_env=self.env)

    def test_recursive_manifest_loader(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest'),
                    verify_openpgp=True,
                    openpgp_env=self.env)
            self.assertTrue(m.openpgp_signed)
            self.assertEqual(m.openpgp_signature.fingerprint, KEY_FINGERPRINT)
            self.assertEqual(m.openpgp_signature.timestamp, SIG_TIMESTAMP)
            self.assertIsNone(m.openpgp_signature.expire_timestamp)
            self.assertEqual(m.openpgp_signature.primary_key_fingerprint, KEY_FINGERPRINT)
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_compressed(self):
        d = tempfile.mkdtemp()
        try:
            with gemato.compression.open_potentially_compressed_path(
                    os.path.join(d, 'Manifest.gz'), 'w') as cf:
                cf.write(SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest.gz'),
                    verify_openpgp=True,
                    openpgp_env=self.env)
            self.assertTrue(m.openpgp_signed)
            self.assertEqual(m.openpgp_signature.fingerprint, KEY_FINGERPRINT)
            self.assertEqual(m.openpgp_signature.timestamp, SIG_TIMESTAMP)
            self.assertIsNone(m.openpgp_signature.expire_timestamp)
            self.assertEqual(m.openpgp_signature.primary_key_fingerprint, KEY_FINGERPRINT)
        finally:
            shutil.rmtree(d)

    def test_cli(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, '.key.asc'), 'wb') as f:
                f.write(PUBLIC_KEY)
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)

            os.mkdir(os.path.join(d, 'eclass'))
            with io.open(os.path.join(d, 'eclass/Manifest'), 'w'):
                pass
            with io.open(os.path.join(d, 'myebuild-0.ebuild'), 'w'):
                pass
            with io.open(os.path.join(d, 'metadata.xml'), 'w'):
                pass

            self.assertEqual(
                    gemato.cli.main(['gemato', 'verify',
                        '--openpgp-key', os.path.join(d, '.key.asc'),
                        '--no-refresh-keys',
                        '--require-signed-manifest', d]),
                    0)
        finally:
            shutil.rmtree(d)


class OpenPGPNoKeyTest(unittest.TestCase):
    """
    Tests performed without correct OpenPGP key set.
    """

    expected_exception = gemato.exceptions.OpenPGPVerificationFailure

    def setUp(self):
        self.env = gemato.openpgp.OpenPGPEnvironment()

    def tearDown(self):
        self.env.close()

    def test_verify_manifest(self):
        with io.StringIO(SIGNED_MANIFEST) as f:
            try:
                self.assertRaises(self.expected_exception,
                        self.env.verify_file, f)
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))

    def test_manifest_load(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            try:
                self.assertRaises(self.expected_exception,
                        m.load, f, openpgp_env=self.env)
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))

    def test_manifest_load_exception_caught(self):
        """
        Test that the Manifest is loaded even if exception is raised.
        """
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            try:
                m.load(f, openpgp_env=self.env)
            except self.expected_exception:
                pass
            except gemato.exceptions.OpenPGPNoImplementation:
                pass
        self.assertIsNotNone(m.find_timestamp())
        self.assertIsNotNone(m.find_path_entry('myebuild-0.ebuild'))
        self.assertFalse(m.openpgp_signed)
        self.assertIsNone(m.openpgp_signature)

    def test_recursive_manifest_loader(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)

            try:
                self.assertRaises(self.expected_exception,
                        gemato.recursiveloader.ManifestRecursiveLoader,
                        os.path.join(d, 'Manifest'),
                        verify_openpgp=True,
                        openpgp_env=self.env)
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_compressed(self):
        d = tempfile.mkdtemp()
        try:
            with gemato.compression.open_potentially_compressed_path(
                    os.path.join(d, 'Manifest.gz'), 'w') as cf:
                cf.write(SIGNED_MANIFEST)

            try:
                self.assertRaises(self.expected_exception,
                        gemato.recursiveloader.ManifestRecursiveLoader,
                        os.path.join(d, 'Manifest.gz'),
                        verify_openpgp=True,
                        openpgp_env=self.env)
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))
        finally:
            shutil.rmtree(d)

    def test_find_top_level_manifest(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)

            self.assertEqual(
                    gemato.find_top_level.find_top_level_manifest(d),
                    os.path.join(d, 'Manifest'))
        finally:
            shutil.rmtree(d)


class OpenPGPExpiredKeyTest(OpenPGPNoKeyTest):
    """
    Tests performed with an expired OpenPGP key.
    """

    expected_exception = gemato.exceptions.OpenPGPExpiredKeyFailure

    def setUp(self):
        self.env = gemato.openpgp.OpenPGPEnvironment()
        try:
            self.env.import_key(io.BytesIO(EXPIRED_PUBLIC_KEY))
        except gemato.exceptions.OpenPGPNoImplementation as e:
            self.env.close()
            raise unittest.SkipTest(str(e))
        except RuntimeError:
            self.env.close()
            raise unittest.SkipTest('Unable to import OpenPGP key')

    def tearDown(self):
        self.env.close()


class OpenPGPRevokedKeyTest(OpenPGPNoKeyTest):
    """
    Tests performed with a revoked OpenPGP key.
    """

    expected_exception = gemato.exceptions.OpenPGPRevokedKeyFailure

    def setUp(self):
        self.env = gemato.openpgp.OpenPGPEnvironment()
        try:
            self.env.import_key(io.BytesIO(REVOKED_PUBLIC_KEY))
        except gemato.exceptions.OpenPGPNoImplementation as e:
            self.env.close()
            raise unittest.SkipTest(str(e))
        except RuntimeError:
            self.env.close()
            raise unittest.SkipTest('Unable to import OpenPGP key')

    def tearDown(self):
        self.env.close()


class OpenPGPContextManagerTest(unittest.TestCase):
    """
    Test the context manager API for OpenPGPEnvironment.
    """

    def test_import_key(self):
        with gemato.openpgp.OpenPGPEnvironment() as env:
            try:
                env.import_key(io.BytesIO(PUBLIC_KEY))
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))

    def test_import_malformed_key(self):
        with gemato.openpgp.OpenPGPEnvironment() as env:
            try:
                self.assertRaises(RuntimeError,
                        env.import_key,
                        io.BytesIO(MALFORMED_PUBLIC_KEY))
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))

    def test_import_no_keys(self):
        with gemato.openpgp.OpenPGPEnvironment() as env:
            try:
                self.assertRaises(RuntimeError,
                        env.import_key,
                        io.BytesIO(b''))
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))

    def test_import_binary_key(self):
        with gemato.openpgp.OpenPGPEnvironment() as env:
            enc = b''.join(PUBLIC_KEY.splitlines()[2:-1])
            try:
                env.import_key(io.BytesIO(base64.b64decode(enc)))
            except gemato.exceptions.OpenPGPNoImplementation as e:
                raise unittest.SkipTest(str(e))

    def test_verify_manifest(self):
        with io.StringIO(SIGNED_MANIFEST) as f:
            with gemato.openpgp.OpenPGPEnvironment() as env:
                try:
                    try:
                        env.import_key(io.BytesIO(PUBLIC_KEY))
                    except RuntimeError:
                        raise unittest.SkipTest('Unable to import OpenPGP key')

                    sig = env.verify_file(f)
                    self.assertEqual(sig.fingerprint, KEY_FINGERPRINT)
                    self.assertEqual(sig.timestamp, SIG_TIMESTAMP)
                    self.assertIsNone(sig.expire_timestamp)
                    self.assertEqual(sig.primary_key_fingerprint, KEY_FINGERPRINT)
                except gemato.exceptions.OpenPGPNoImplementation as e:
                    raise unittest.SkipTest(str(e))

    def test_double_close(self):
        with gemato.openpgp.OpenPGPEnvironment() as env:
            env.close()

    def test_home_after_close(self):
        with gemato.openpgp.OpenPGPEnvironment() as env:
            env.close()
            with self.assertRaises(RuntimeError):
                env.home


class OpenPGPPrivateKeyTest(unittest.TestCase):
    """
    Tests performed with the private key available.
    """

    TEST_STRING = u'The quick brown fox jumps over the lazy dog'

    def setUp(self):
        self.env = gemato.openpgp.OpenPGPEnvironment()
        try:
            self.env.import_key(io.BytesIO(PRIVATE_KEY))
        except gemato.exceptions.OpenPGPNoImplementation as e:
            self.env.close()
            raise unittest.SkipTest(str(e))
        except RuntimeError:
            self.env.close()
            raise unittest.SkipTest('Unable to import OpenPGP key')

    def tearDown(self):
        self.env.close()

    def test_verify_manifest(self):
        with io.StringIO(SIGNED_MANIFEST) as f:
            sig = self.env.verify_file(f)
            self.assertEqual(sig.fingerprint, KEY_FINGERPRINT)
            self.assertEqual(sig.timestamp, SIG_TIMESTAMP)
            self.assertIsNone(sig.expire_timestamp)
            self.assertEqual(sig.primary_key_fingerprint, KEY_FINGERPRINT)

    def test_sign_data(self):
        with io.StringIO(self.TEST_STRING) as f:
            with io.StringIO() as wf:
                self.env.clear_sign_file(f, wf)
                wf.seek(0)
                self.env.verify_file(wf)

    def test_sign_data_keyid(self):
        with io.StringIO(self.TEST_STRING) as f:
            with io.StringIO() as wf:
                self.env.clear_sign_file(f, wf, keyid=PRIVATE_KEY_ID)
                wf.seek(0)
                self.env.verify_file(wf)

    def test_dump_signed_manifest(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            m.load(f, openpgp_env=self.env)
        with io.StringIO() as f:
            m.dump(f, openpgp_env=self.env)
            f.seek(0)
            m.load(f, openpgp_env=self.env)
        self.assertTrue(m.openpgp_signed)
        self.assertIsNotNone(m.openpgp_signature)

    def test_dump_signed_manifest_keyid(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            m.load(f, openpgp_env=self.env)
        with io.StringIO() as f:
            m.dump(f, openpgp_keyid=PRIVATE_KEY_ID, openpgp_env=self.env)
            f.seek(0)
            m.load(f, openpgp_env=self.env)
        self.assertTrue(m.openpgp_signed)
        self.assertIsNotNone(m.openpgp_signature)

    def test_dump_force_signed_manifest(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            m.load(f, verify_openpgp=False, openpgp_env=self.env)
        self.assertFalse(m.openpgp_signed)
        self.assertIsNone(m.openpgp_signature)
        with io.StringIO() as f:
            m.dump(f, sign_openpgp=True, openpgp_env=self.env)
            f.seek(0)
            m.load(f, openpgp_env=self.env)
        self.assertTrue(m.openpgp_signed)
        self.assertIsNotNone(m.openpgp_signature)

    def test_dump_force_unsigned_manifest(self):
        m = gemato.manifest.ManifestFile()
        with io.StringIO(SIGNED_MANIFEST) as f:
            m.load(f, openpgp_env=self.env)
        self.assertTrue(m.openpgp_signed)
        with io.StringIO() as f:
            m.dump(f, sign_openpgp=False, openpgp_env=self.env)
            f.seek(0)
            m.load(f, openpgp_env=self.env)
        self.assertFalse(m.openpgp_signed)
        self.assertIsNone(m.openpgp_signature)

    def test_recursive_manifest_loader_save_manifest(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest'),
                    verify_openpgp=True,
                    openpgp_env=self.env)
            self.assertTrue(m.openpgp_signed)

            m.save_manifest('Manifest')
            m2 = gemato.manifest.ManifestFile()
            with io.open(os.path.join(d, 'Manifest'), 'r') as f:
                m2.load(f, openpgp_env=self.env)
            self.assertTrue(m2.openpgp_signed)
            self.assertIsNotNone(m.openpgp_signature)
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_save_manifest_compressed(self):
        d = tempfile.mkdtemp()
        try:
            with gemato.compression.open_potentially_compressed_path(
                    os.path.join(d, 'Manifest.gz'), 'w') as cf:
                cf.write(SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest.gz'),
                    verify_openpgp=True,
                    openpgp_env=self.env)
            self.assertTrue(m.openpgp_signed)

            m.save_manifest('Manifest.gz')
            m2 = gemato.manifest.ManifestFile()
            with gemato.compression.open_potentially_compressed_path(
                    os.path.join(d, 'Manifest.gz'), 'r') as cf:
                m2.load(cf, openpgp_env=self.env)
            self.assertTrue(m2.openpgp_signed)
            self.assertIsNotNone(m.openpgp_signature)
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_save_manifest_force_sign(self):
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest'),
                    verify_openpgp=False,
                    sign_openpgp=True,
                    openpgp_env=self.env)
            self.assertFalse(m.openpgp_signed)
            self.assertIsNone(m.openpgp_signature)

            m.save_manifest('Manifest')
            m2 = gemato.manifest.ManifestFile()
            with io.open(os.path.join(d, 'Manifest'), 'r') as f:
                m2.load(f, openpgp_env=self.env)
            self.assertTrue(m2.openpgp_signed)
            self.assertIsNotNone(m2.openpgp_signature)
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_save_manifest_compressed_force_sign(self):
        d = tempfile.mkdtemp()
        try:
            with gemato.compression.open_potentially_compressed_path(
                    os.path.join(d, 'Manifest.gz'), 'w') as cf:
                cf.write(SIGNED_MANIFEST)

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest.gz'),
                    verify_openpgp=False,
                    sign_openpgp=True,
                    openpgp_env=self.env)
            self.assertFalse(m.openpgp_signed)
            self.assertIsNone(m.openpgp_signature)

            m.save_manifest('Manifest.gz')
            m2 = gemato.manifest.ManifestFile()
            with gemato.compression.open_potentially_compressed_path(
                    os.path.join(d, 'Manifest.gz'), 'r') as cf:
                m2.load(cf, openpgp_env=self.env)
            self.assertTrue(m2.openpgp_signed)
            self.assertIsNotNone(m2.openpgp_signature)
        finally:
            shutil.rmtree(d)

    def test_recursive_manifest_loader_save_submanifest_force_sign(self):
        """
        Test that sub-Manifests are not signed.
        """
        d = tempfile.mkdtemp()
        try:
            with io.open(os.path.join(d, 'Manifest'), 'w') as f:
                f.write(SIGNED_MANIFEST)
            os.mkdir(os.path.join(d, 'eclass'))
            with io.open(os.path.join(d, 'eclass/Manifest'), 'w'):
                pass

            m = gemato.recursiveloader.ManifestRecursiveLoader(
                    os.path.join(d, 'Manifest'),
                    verify_openpgp=False,
                    sign_openpgp=True,
                    openpgp_env=self.env)
            self.assertFalse(m.openpgp_signed)
            self.assertIsNone(m.openpgp_signature)

            m.load_manifest('eclass/Manifest')
            m.save_manifest('eclass/Manifest')

            m2 = gemato.manifest.ManifestFile()
            with io.open(os.path.join(d, 'eclass/Manifest'), 'r') as f:
                m2.load(f, openpgp_env=self.env)
            self.assertFalse(m2.openpgp_signed)
            self.assertIsNone(m2.openpgp_signature)
        finally:
            shutil.rmtree(d)



# gemato: Utility function tests
# vim:fileencoding=utf-8
# (c) 2017 Michał Górny
# Licensed under the terms of 2-clause BSD license

import unittest

import gemato.util


class UtilityTestCase(unittest.TestCase):
    def test_path_starts_with(self):
        self.assertTrue(gemato.util.path_starts_with("", ""))
        self.assertTrue(gemato.util.path_starts_with("foo", ""))
        self.assertTrue(gemato.util.path_starts_with("foo/", ""))
        self.assertTrue(gemato.util.path_starts_with("foo/bar", ""))
        self.assertTrue(gemato.util.path_starts_with("bar", ""))
        self.assertTrue(gemato.util.path_starts_with("bar/", ""))
        self.assertTrue(gemato.util.path_starts_with("bar/bar", ""))
        self.assertTrue(gemato.util.path_starts_with("foo", "foo"))
        self.assertTrue(gemato.util.path_starts_with("foo/", "foo"))
        self.assertTrue(gemato.util.path_starts_with("foo/bar", "foo"))
        self.assertFalse(gemato.util.path_starts_with("bar", "foo"))
        self.assertFalse(gemato.util.path_starts_with("fooo", "foo"))
        self.assertFalse(gemato.util.path_starts_with("foo.", "foo"))
        self.assertTrue(gemato.util.path_starts_with("foo", "foo/"))
        self.assertTrue(gemato.util.path_starts_with("foo/", "foo/"))
        self.assertTrue(gemato.util.path_starts_with("foo/bar", "foo/bar/"))

    def test_path_inside_dir(self):
        self.assertFalse(gemato.util.path_inside_dir("", ""))
        self.assertTrue(gemato.util.path_inside_dir("foo", ""))
        self.assertTrue(gemato.util.path_inside_dir("foo/", ""))
        self.assertTrue(gemato.util.path_inside_dir("foo/bar", ""))
        self.assertTrue(gemato.util.path_inside_dir("bar", ""))
        self.assertTrue(gemato.util.path_inside_dir("bar/", ""))
        self.assertTrue(gemato.util.path_inside_dir("bar/bar", ""))
        self.assertFalse(gemato.util.path_inside_dir("foo", "foo"))
        self.assertFalse(gemato.util.path_inside_dir("foo/", "foo"))
        self.assertTrue(gemato.util.path_inside_dir("foo/bar", "foo"))
        self.assertFalse(gemato.util.path_inside_dir("bar", "foo"))
        self.assertFalse(gemato.util.path_inside_dir("fooo", "foo"))
        self.assertFalse(gemato.util.path_inside_dir("foo.", "foo"))
        self.assertFalse(gemato.util.path_inside_dir("foo", "foo/"))
        self.assertFalse(gemato.util.path_inside_dir("foo/", "foo/"))
        self.assertFalse(gemato.util.path_inside_dir("foo/bar", "foo/bar/"))



# gemato: Verification tests
# vim:fileencoding=utf-8
# (c) 2017 Michał Górny
# Licensed under the terms of 2-clause BSD license

import io
import os
import os.path
import socket
import stat
import tempfile
import unittest

import gemato.exceptions
import gemato.manifest
import gemato.verify


class NonExistingFileVerificationTest(unittest.TestCase):
    def setUp(self):
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        os.rmdir(self.dir)

    def test_get_file_metadata(self):
        self.assertEqual(list(gemato.verify.get_file_metadata(
            os.path.join(self.dir, 'test'), hashes=[])),
            [False])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0'))
        self.assertEqual(gemato.verify.verify_path(os.path.join(self.dir, e.path), e),
                (False, [('__exists__', True, False)]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', 'test'))
        self.assertEqual(gemato.verify.verify_path(os.path.join(self.dir, e.path), e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(os.path.join(self.dir, 'test'), None),
                (True, []))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA('test', 0, {})
        self.assertRaises(gemato.exceptions.ManifestInvalidPath,
                gemato.verify.update_entry_for_path,
                os.path.join(self.dir, 'test'), e)


class DirectoryVerificationTest(unittest.TestCase):
    def setUp(self):
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        os.rmdir(self.dir)

    def test_get_file_metadata(self):
        st = os.stat(self.dir)
        self.assertEqual(list(gemato.verify.get_file_metadata(
            self.dir, hashes=[])),
            [True, st.st_dev, (stat.S_IFDIR, 'directory')])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.dir), '0'))
        self.assertEqual(gemato.verify.verify_path(self.dir, e),
                (False, [('__type__', 'regular file', 'directory')]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.dir)))
        self.assertEqual(gemato.verify.verify_path(self.dir, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.dir, None),
                (False, [('__exists__', False, True)]))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.dir), 0, {})
        self.assertRaises(gemato.exceptions.ManifestInvalidPath,
                gemato.verify.update_entry_for_path, self.dir, e)


class CharacterDeviceVerificationTest(unittest.TestCase):
    def setUp(self):
        self.path = '/dev/null'

    def test_get_file_metadata(self):
        st = os.stat(self.path)
        self.assertEqual(list(gemato.verify.get_file_metadata(
            self.path, hashes=[])),
            [True, st.st_dev, (stat.S_IFCHR, 'character device')])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__type__', 'regular file', 'character device')]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.path)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.path, None),
                (False, [('__exists__', False, True)]))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertRaises(gemato.exceptions.ManifestInvalidPath,
                gemato.verify.update_entry_for_path, self.path, e)


class NamedPipeVerificationTest(unittest.TestCase):
    def setUp(self):
        self.dir = tempfile.mkdtemp()
        self.path = os.path.join(self.dir, 'test')
        os.mkfifo(self.path)

    def tearDown(self):
        os.unlink(self.path)
        os.rmdir(self.dir)

    def test_get_file_metadata(self):
        st = os.stat(self.path)
        self.assertEqual(list(gemato.verify.get_file_metadata(
            self.path, hashes=[])),
            [True, st.st_dev, (stat.S_IFIFO, 'named pipe')])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__type__', 'regular file', 'named pipe')]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.path)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.path, None),
                (False, [('__exists__', False, True)]))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertRaises(gemato.exceptions.ManifestInvalidPath,
                gemato.verify.update_entry_for_path, self.path, e)


class UNIXSocketVerificationTest(unittest.TestCase):
    def setUp(self):
        self.dir = tempfile.mkdtemp()
        self.path = os.path.join(self.dir, 'test')
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.sock.bind(self.path)
        self.sock.listen(1)

    def tearDown(self):
        self.sock.close()
        os.unlink(self.path)
        os.rmdir(self.dir)

    def test_get_file_metadata(self):
        st = os.stat(self.path)
        self.assertEqual(list(gemato.verify.get_file_metadata(
            self.path, hashes=[])),
            [True, st.st_dev, (stat.S_IFSOCK, 'UNIX socket')])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__type__', 'regular file', 'UNIX socket')]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.path)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.path, None),
                (False, [('__exists__', False, True)]))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertRaises(gemato.exceptions.ManifestInvalidPath,
                gemato.verify.update_entry_for_path, self.path, e)


class EmptyFileVerificationTest(unittest.TestCase):
    def setUp(self):
        self.f = tempfile.NamedTemporaryFile()
        self.path = self.f.name

    def tearDown(self):
        self.f.close()

    def test_get_file_metadata(self):
        st = os.stat(self.path)
        self.assertEqual(list(gemato.verify.get_file_metadata(
            self.path, hashes=['MD5', 'SHA1'])),
            [True, st.st_dev, (stat.S_IFREG, 'regular file'),
                0, st.st_mtime, {
                    'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                    '__size__': 0,
                }])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0',
                    'MD5', 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testWrongSizeDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '5'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__size__', 5, 0)]))

    def testWrongSingleChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('MD5', '9e107d9d372bb6826bd81d3542a419d6', 'd41d8cd98f00b204e9800998ecf8427e')]))

    def testWrongAllChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('MD5', '9e107d9d372bb6826bd81d3542a419d6', 'd41d8cd98f00b204e9800998ecf8427e'),
                    ('SHA1', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12', 'da39a3ee5e6b4b0d3255bfef95601890afd80709')]))

    def testWrongAllChecksumAndSizeDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '39',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__size__', 39, 0),
                    ('MD5', '9e107d9d372bb6826bd81d3542a419d6', 'd41d8cd98f00b204e9800998ecf8427e'),
                    ('SHA1', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12', 'da39a3ee5e6b4b0d3255bfef95601890afd80709')]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.path)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.path, None),
                (False, [('__exists__', False, True)]))

    def testCrossFilesystem(self):
        try:
            st = os.stat('/proc')
        except OSError:
            raise unittest.SkipTest('Unable to stat /proc')

        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertRaises(gemato.exceptions.ManifestCrossDevice,
                gemato.verify.verify_path, self.path, e,
                expected_dev=st.st_dev)

    def testCrossFilesystemAssert(self):
        try:
            st = os.stat('/proc')
        except OSError:
            raise unittest.SkipTest('Unable to stat /proc')

        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertRaises(gemato.exceptions.ManifestCrossDevice,
                gemato.verify.verify_path, self.path, e,
                expected_dev=st.st_dev)

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertFalse(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {})

    def test_update_with_hashes(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1']))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {
                'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
            })

    def test_update_with_hashes_from_manifest(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {'MD5': '', 'SHA1': ''})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {
                'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
            })

    def test_update_with_hashes_unchanged(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {
                    'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                })
        self.assertFalse(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1']))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {
                'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
            })

    def test_update_cross_filesystem(self):
        try:
            st = os.stat('/proc')
        except OSError:
            raise unittest.SkipTest('Unable to stat /proc')

        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertRaises(gemato.exceptions.ManifestCrossDevice,
                gemato.verify.update_entry_for_path, self.path, e,
                expected_dev=st.st_dev)

    def test_update_MISC(self):
        e = gemato.manifest.ManifestEntryMISC(
                os.path.basename(self.path), 0, {})
        self.assertFalse(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {})

    def test_update_IGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE(
                os.path.basename(self.path))
        self.assertRaises(AssertionError,
                gemato.verify.update_entry_for_path, self.path, e)

    def test_update_AUX(self):
        e = gemato.manifest.ManifestEntryAUX(
                os.path.basename(self.path), 0, {})
        self.assertFalse(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path,
                os.path.join('files', os.path.basename(self.path)))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {})

    def test_wrong_checksum_DATA_with_old_mtime(self):
        """
        Test whether the checksums are verified if last mtime is older
        than the current one.
        """
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(
                gemato.verify.verify_path(self.path, e,
                    last_mtime=0),
                (False, [('MD5', '9e107d9d372bb6826bd81d3542a419d6', 'd41d8cd98f00b204e9800998ecf8427e')]))

    def test_wrong_checksum_DATA_with_new_mtime(self):
        """
        Test whether the checksums are verified if last mtime indicates
        that the file did not change (with st_size == 0).
        """
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        st = os.stat(self.path)
        self.assertEqual(
                gemato.verify.verify_path(self.path, e,
                    last_mtime=st.st_mtime),
                (False, [('MD5', '9e107d9d372bb6826bd81d3542a419d6', 'd41d8cd98f00b204e9800998ecf8427e')]))

    def test_update_with_hashes_and_old_mtime(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {
                    'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                })
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1'], last_mtime=0))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {
                'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
            })

    def test_update_with_hashes_and_new_mtime(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {
                    'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                })
        st = os.stat(self.path)
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1'], last_mtime=st.st_mtime))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 0)
        self.assertDictEqual(e.checksums, {
                'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
                'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
            })

    def test_None_with_mtime(self):
        """
        Test that mtime does not cause stray files to go unnoticed.
        """
        st = os.stat(self.path)
        self.assertEqual(
                gemato.verify.verify_path(self.path, None,
                    last_mtime=st.st_mtime),
                (False, [('__exists__', False, True)]))


class NonEmptyFileVerificationTest(unittest.TestCase):
    def setUp(self):
        TEST_STRING = b'The quick brown fox jumps over the lazy dog'
        self.f = tempfile.NamedTemporaryFile()
        self.f.write(TEST_STRING)
        self.f.flush()
        self.path = self.f.name

    def tearDown(self):
        self.f.close()

    def test_get_file_metadata(self):
        st = os.stat(self.path)
        self.assertEqual(list(gemato.verify.get_file_metadata(
            self.path, hashes=['MD5', 'SHA1'])),
            [True, st.st_dev, (stat.S_IFREG, 'regular file'),
                st.st_size, st.st_mtime, {
                    'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1': '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12',
                    '__size__': 43,
                }])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '43'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '43',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testWrongSizeDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__size__', 0, 43)]))

    def testWrongSingleChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '43',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12')]))

    def testWrongAllChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '43',
                    'MD5', 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('MD5', 'd41d8cd98f00b204e9800998ecf8427e', '9e107d9d372bb6826bd81d3542a419d6'),
                    ('SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12')]))

    def testWrongAllChecksumAndSizeDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '0',
                    'MD5', 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__size__', 0, 43)]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.path)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.path, None),
                (False, [('__exists__', False, True)]))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 43)
        self.assertDictEqual(e.checksums, {})

    def test_update_with_hashes(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1']))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 43)
        self.assertDictEqual(e.checksums, {
                'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                'SHA1': '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12',
            })

    def test_update_with_hashes_from_manifest(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {'MD5': '', 'SHA1': ''})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 43)
        self.assertDictEqual(e.checksums, {
                'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                'SHA1': '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12',
            })

    def test_wrong_checksum_DATA_with_old_mtime(self):
        """
        Test whether the checksums are verified if last mtime is older
        than the current one.
        """
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '43',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(
                gemato.verify.verify_path(self.path, e,
                    last_mtime=0),
                (False, [('SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709', '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12')]))

    def test_wrong_checksum_DATA_with_new_mtime(self):
        """
        Test whether the checksums are verified if last mtime indicates
        that the file did not change.
        """
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '43',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        st = os.stat(self.path)
        self.assertEqual(
                gemato.verify.verify_path(self.path, e,
                    last_mtime=st.st_mtime),
                (True, []))

    def test_wrong_checksum_DATA_with_new_mtime_and_wrong_size(self):
        """
        Test whether the checksums are verified if last mtime indicates
        that the file did not change but size is different.
        """
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '33',
                    'MD5', '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        st = os.stat(self.path)
        self.assertEqual(
                gemato.verify.verify_path(self.path, e,
                    last_mtime=st.st_mtime),
                (False, [('__size__', 33, 43)]))

    def test_update_with_hashes_and_old_mtime(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {
                    'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                })
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1'], last_mtime=0))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 43)
        self.assertDictEqual(e.checksums, {
                'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                'SHA1': '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12',
            })

    def test_update_with_hashes_and_new_mtime(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 43, {
                    'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                })
        st = os.stat(self.path)
        self.assertFalse(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1'], last_mtime=st.st_mtime))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 43)
        self.assertDictEqual(e.checksums, {
                'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
            })

    def test_update_with_hashes_and_new_mtime_but_wrong_size(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 33, {
                    'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                    'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
                })
        st = os.stat(self.path)
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1'], last_mtime=st.st_mtime))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, 43)
        self.assertDictEqual(e.checksums, {
                'MD5': '9e107d9d372bb6826bd81d3542a419d6',
                'SHA1': '2fd4e1c67a2d28fced849ee1bb76e7391b93eb12',
            })

    def test_None_with_mtime(self):
        """
        Test that mtime does not cause stray files to go unnoticed.
        """
        st = os.stat(self.path)
        self.assertEqual(
                gemato.verify.verify_path(self.path, None,
                    last_mtime=st.st_mtime),
                (False, [('__exists__', False, True)]))


class SymbolicLinkVerificationTest(NonEmptyFileVerificationTest):
    """
    A variant of regular file test using symlink.
    """

    def setUp(self):
        TEST_STRING = b'The quick brown fox jumps over the lazy dog'
        self.dir = tempfile.mkdtemp()
        self.real_path = os.path.join(self.dir, 'real')
        self.path = os.path.join(self.dir, 'symlink')
        with io.open(self.real_path, 'wb') as f:
            f.write(TEST_STRING)
        os.symlink('real', self.path)

    def tearDown(self):
        os.unlink(self.path)
        os.unlink(self.real_path)
        os.rmdir(self.dir)


class SymbolicLinkDirectoryVerificationTest(DirectoryVerificationTest):
    """
    A variant of directory test using symlink.
    """

    def setUp(self):
        self.top_dir = tempfile.mkdtemp()
        self.real_dir = os.path.join(self.top_dir, 'real')
        self.dir = os.path.join(self.top_dir, 'symlink')
        os.mkdir(self.real_dir)
        os.symlink('real', self.dir)

    def tearDown(self):
        os.unlink(self.dir)
        os.rmdir(self.real_dir)
        os.rmdir(self.top_dir)


class BrokenSymbolicLinkVerificationTest(NonExistingFileVerificationTest):
    def setUp(self):
        self.dir = tempfile.mkdtemp()
        self.path = os.path.join(self.dir, 'test')
        os.symlink('broken', self.path)

    def tearDown(self):
        os.unlink(self.path)
        os.rmdir(self.dir)


class ProcFileVerificationTest(unittest.TestCase):
    """
    Attempt to verify a file from /proc to verify that we can handle
    filesystems that do not report st_size.
    """

    def setUp(self):
        self.path = '/proc/version'
        try:
            with io.open(self.path, 'rb') as f:
                data = f.read()
                st = os.fstat(f.fileno())
        except:
            raise unittest.SkipTest('{} not readable'.format(self.path))

        if st.st_size != 0:
            raise unittest.SkipTest('{} st_size is not 0'.format(self.path))

        self.size = len(data)
        if self.size == 0:
            raise unittest.SkipTest('{} empty'.format(self.path))
        self.md5 = gemato.hash.hash_bytes(data, 'md5')
        self.sha1 = gemato.hash.hash_bytes(data, 'sha1')

    def test_get_file_metadata(self):
        st = os.stat(self.path)
        metadata = list(gemato.verify.get_file_metadata(
            self.path, hashes=['MD5', 'SHA1']))
        # mtime is not meaningful on procfs, and changes with every stat
        metadata[4] = 0
        self.assertEqual(metadata,
            [True, st.st_dev, (stat.S_IFREG, 'regular file'),
                st.st_size, 0, {
                    'MD5': self.md5,
                    'SHA1': self.sha1,
                    '__size__': self.size,
                }])

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), str(self.size)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), str(self.size),
                    'MD5', self.md5,
                    'SHA1', self.sha1))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testWrongSizeDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '47474'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__size__', 47474, self.size)]))

    def testWrongSingleChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), self.size,
                    'MD5', self.md5,
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709', self.sha1)]))

    def testWrongAllChecksumDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), self.size,
                    'MD5', 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('MD5', 'd41d8cd98f00b204e9800998ecf8427e', self.md5),
                    ('SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709', self.sha1)]))

    def testWrongAllChecksumAndSizeDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', os.path.basename(self.path), '47474',
                    'MD5', 'd41d8cd98f00b204e9800998ecf8427e',
                    'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (False, [('__size__', 47474, self.size),
                    ('MD5', 'd41d8cd98f00b204e9800998ecf8427e', self.md5),
                    ('SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709', self.sha1)]))

    def testIGNORE(self):
        e = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', os.path.basename(self.path)))
        self.assertEqual(gemato.verify.verify_path(self.path, e),
                (True, []))

    def testNone(self):
        self.assertEqual(gemato.verify.verify_path(self.path, None),
                (False, [('__exists__', False, True)]))

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, self.size)
        self.assertDictEqual(e.checksums, {})

    def test_update_with_hashes(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e,
                    ['MD5', 'SHA1']))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, self.size)
        self.assertDictEqual(e.checksums, {
                'MD5': self.md5,
                'SHA1': self.sha1,
            })

    def test_update_with_hashes_from_manifest(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {'MD5': '', 'SHA1': ''})
        self.assertTrue(
                gemato.verify.update_entry_for_path(self.path, e))
        self.assertEqual(e.path, os.path.basename(self.path))
        self.assertEqual(e.size, self.size)
        self.assertDictEqual(e.checksums, {
                'MD5': self.md5,
                'SHA1': self.sha1,
            })


class UnreadableFileVerificationTest(unittest.TestCase):
    def setUp(self):
        self.dir = tempfile.mkdtemp()
        self.path = os.path.join(self.dir, 'test')
        with io.open(self.path, 'w'):
            pass
        os.chmod(self.path, 0)

    def tearDown(self):
        os.unlink(self.path)
        os.rmdir(self.dir)

    def test_get_file_metadata(self):
        with self.assertRaises(OSError):
            list(gemato.verify.get_file_metadata(
                os.path.join(self.dir, self.path), []))

    def testDATA(self):
        e = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0'))
        self.assertRaises(OSError, gemato.verify.verify_path,
                os.path.join(self.dir, e.path), e)

    def test_update(self):
        e = gemato.manifest.ManifestEntryDATA(
                os.path.basename(self.path), 0, {})
        self.assertRaises(OSError,
                gemato.verify.update_entry_for_path, self.path, e)


class EntryCompatibilityVerificationTest(unittest.TestCase):
    def test_matching_entry(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (True, []))

    def test_compatible_types(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryEBUILD.from_list(
                ('EBUILD', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (True, []))

    def test_compatible_types_AUX(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'files/test.patch', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryAUX.from_list(
                ('AUX', 'test.patch', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (True, []))

    def test_compatible_types_MANIFEST(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryMANIFEST.from_list(
                ('MANIFEST', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (True, []))

    def test_incompatible_types_DATA_MISC(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryMISC.from_list(
                ('MISC', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (False, [('__type__', 'DATA', 'MISC')]))

    def test_incompatible_types_DATA_IGNORE(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryIGNORE.from_list(
                ('IGNORE', 'test'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (False, [('__type__', 'DATA', 'IGNORE')]))

    def test_incompatible_types_DATA_DIST(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryDIST.from_list(
                ('DIST', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (False, [('__type__', 'DATA', 'DIST')]))

    def test_mismatched_size(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '32', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (False, [('__size__', 0, 32)]))

    def test_mismatched_md5(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', '9e107d9d372bb6826bd81d3542a419d6'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (False, [('MD5', 'd41d8cd98f00b204e9800998ecf8427e', '9e107d9d372bb6826bd81d3542a419d6')]))

    def test_different_hashsets(self):
        e1 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e'))
        e2 = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA', 'test', '0', 'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709'))
        self.assertEqual(gemato.verify.verify_entry_compatibility(e1, e2),
                (True, [('MD5', 'd41d8cd98f00b204e9800998ecf8427e', None),
                        ('SHA1', None, 'da39a3ee5e6b4b0d3255bfef95601890afd80709')]))



# gemato: Manifest file support tests
# vim:fileencoding=utf-8
# (c) 2017-2018 Michał Górny
# Licensed under the terms of 2-clause BSD license

import datetime
import io
import unittest

import gemato.exceptions
import gemato.manifest


TEST_MANIFEST = u'''
TIMESTAMP 2017-10-22T18:06:41Z
MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
IGNORE local
DATA myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
DATA foo.txt 0
'''

TEST_DEPRECATED_MANIFEST = u'''
EBUILD myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
AUX test.patch 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
'''


class ManifestTest(unittest.TestCase):
    """
    Basic tests for Manifest processing.
    """

    def test_load(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))

    def test_load_deprecated(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_DEPRECATED_MANIFEST))

    def test_load_via_ctor(self):
        gemato.manifest.ManifestFile(io.StringIO(TEST_MANIFEST))

    def test_load_and_dump(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        outf = io.StringIO()
        m.dump(outf)
        self.assertEqual(outf.getvalue().strip(), TEST_MANIFEST.strip())

    def test_load_and_dump_deprecated(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_DEPRECATED_MANIFEST))
        outf = io.StringIO()
        m.dump(outf)
        self.assertEqual(outf.getvalue().strip(), TEST_DEPRECATED_MANIFEST.strip())

    def test_find_timestamp(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        self.assertEqual(m.find_timestamp().ts,
                datetime.datetime(2017, 10, 22, 18, 6, 41))

    def test_find_timestamp_none(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(u''))
        self.assertIsNone(m.find_timestamp())

    def test_find_path_entry(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        self.assertIsNone(m.find_path_entry('2017-10-22T18:06:41Z'))
        self.assertEqual(m.find_path_entry('eclass/Manifest').path, 'eclass/Manifest')
        self.assertIsNone(m.find_path_entry('eclass'))
        self.assertEqual(m.find_path_entry('local').path, 'local')
        self.assertEqual(m.find_path_entry('local/foo').path, 'local')
        self.assertIsNone(m.find_path_entry('locale'))
        self.assertEqual(m.find_path_entry('myebuild-0.ebuild').path, 'myebuild-0.ebuild')
        self.assertEqual(m.find_path_entry('metadata.xml').path, 'metadata.xml')
        self.assertIsNone(m.find_path_entry('mydistfile.tar.gz'))

    def test_find_path_entry_AUX(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_DEPRECATED_MANIFEST))
        self.assertIsNone(m.find_path_entry('test.patch'))
        self.assertEqual(m.find_path_entry('files/test.patch').aux_path, 'test.patch')

    def test_find_dist_entry(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        self.assertIsNone(m.find_dist_entry('myebuild-0.ebuild'))
        self.assertEqual(m.find_dist_entry('mydistfile.tar.gz').path, 'mydistfile.tar.gz')

    def test_find_manifests_for_path(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        self.assertListEqual(list(m.find_manifests_for_path('foo')), [])
        self.assertListEqual(list(m.find_manifests_for_path('eclass')), [])
        self.assertListEqual(list(m.find_manifests_for_path('eclass/foo.eclass')),
                [m.find_path_entry('eclass/Manifest')])

    def test_multiple_load(self):
        """
        Test that calling load() multiple times overwrites previously
        loaded data.
        """
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        m.load(io.StringIO(TEST_DEPRECATED_MANIFEST))
        self.assertIsNone(m.find_timestamp())
        self.assertIsNone(m.find_path_entry('eclass/Manifest'))

    def test_sorted(self):
        m = gemato.manifest.ManifestFile()
        m.load(io.StringIO(TEST_MANIFEST))
        with io.StringIO() as f:
            m.dump(f, sort=True)
            self.assertEqual(f.getvalue(), u'''
DATA foo.txt 0
DATA myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
IGNORE local
MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
TIMESTAMP 2017-10-22T18:06:41Z
'''.lstrip())


class ManifestEntryTest(unittest.TestCase):
    """
    Basic tests for Manifest entries.
    """

    file_vals = ('test', '0', 'MD5', 'd41d8cd98f00b204e9800998ecf8427e',
            'SHA1', 'da39a3ee5e6b4b0d3255bfef95601890afd80709')
    exp_cksums = {
        'MD5': 'd41d8cd98f00b204e9800998ecf8427e',
        'SHA1': 'da39a3ee5e6b4b0d3255bfef95601890afd80709',
    }

    def test_TIMESTAMP(self):
        self.assertEqual(gemato.manifest.ManifestEntryTIMESTAMP.from_list(
                    ('TIMESTAMP', '2010-01-01T11:12:13Z')).ts,
                datetime.datetime(2010, 1, 1, 11, 12, 13))
        self.assertListEqual(list(gemato.manifest.ManifestEntryTIMESTAMP(
                    datetime.datetime(2010, 1, 1, 11, 12, 13)).to_list()),
                ['TIMESTAMP', '2010-01-01T11:12:13Z'])

    def test_MANIFEST(self):
        m = gemato.manifest.ManifestEntryMANIFEST.from_list(
                ('MANIFEST',) + self.file_vals)
        self.assertEqual(m.path, 'test')
        self.assertEqual(m.size, 0)
        self.assertDictEqual(m.checksums, self.exp_cksums)
        self.assertListEqual(list(m.to_list()),
                ['MANIFEST'] + list(self.file_vals))

    def test_IGNORE(self):
        self.assertEqual(gemato.manifest.ManifestEntryIGNORE.from_list(
                    ('IGNORE', 'test')).path,
                'test')
        self.assertListEqual(list(gemato.manifest.ManifestEntryIGNORE('test').to_list()),
                ['IGNORE', 'test'])

    def test_DATA(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(
                ('DATA',) + self.file_vals)
        self.assertEqual(m.path, 'test')
        self.assertEqual(m.size, 0)
        self.assertDictEqual(m.checksums, self.exp_cksums)
        self.assertListEqual(list(m.to_list()),
                ['DATA'] + list(self.file_vals))

    def test_MISC(self):
        m = gemato.manifest.ManifestEntryMISC.from_list(
                ('MISC',) + self.file_vals)
        self.assertEqual(m.path, 'test')
        self.assertEqual(m.size, 0)
        self.assertDictEqual(m.checksums, self.exp_cksums)
        self.assertListEqual(list(m.to_list()),
                ['MISC'] + list(self.file_vals))

    def test_DIST(self):
        m = gemato.manifest.ManifestEntryDIST.from_list(
                ('DIST',) + self.file_vals)
        self.assertEqual(m.path, 'test')
        self.assertEqual(m.size, 0)
        self.assertDictEqual(m.checksums, self.exp_cksums)
        self.assertListEqual(list(m.to_list()),
                ['DIST'] + list(self.file_vals))

    def test_EBUILD(self):
        m = gemato.manifest.ManifestEntryEBUILD.from_list(
                ('EBUILD',) + self.file_vals)
        self.assertEqual(m.path, 'test')
        self.assertEqual(m.size, 0)
        self.assertDictEqual(m.checksums, self.exp_cksums)
        self.assertListEqual(list(m.to_list()),
                ['EBUILD'] + list(self.file_vals))

    def test_AUX(self):
        m = gemato.manifest.ManifestEntryAUX.from_list(
                ('AUX',) + self.file_vals)
        self.assertEqual(m.aux_path, 'test')
        self.assertEqual(m.path, 'files/test')
        self.assertEqual(m.size, 0)
        self.assertDictEqual(m.checksums, self.exp_cksums)
        self.assertListEqual(list(m.to_list()),
                ['AUX'] + list(self.file_vals))

    def test_timestamp_invalid(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryTIMESTAMP.from_list, ('TIMESTAMP',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryTIMESTAMP.from_list, ('TIMESTAMP', ''))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryTIMESTAMP.from_list, ('TIMESTAMP', '2017-10-22T18:06:41+02:00'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryTIMESTAMP.from_list, ('TIMESTAMP', '2017-10-22T18:06:41'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryTIMESTAMP.from_list, ('TIMESTAMP', '2017-10-22', '18:06:41Z'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryTIMESTAMP.from_list, ('TIMESTAMP', '20171022T180641Z'))

    def test_path_invalid(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', '', '0'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', '/foo', '0'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryIGNORE.from_list, ('IGNORE', '',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryIGNORE.from_list, ('IGNORE', '/foo',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', '',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', '/foo',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', '',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', '/foo',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', '',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', '/foo',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', 'foo/bar.gz',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', '',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', '/foo',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', '',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', '/foo',))

    def test_size_invalid(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', 'foo', 'asdf'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', 'foo', '5ds'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', 'foo', '-5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', 'foo', 'asdf'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', 'foo', '5ds'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', 'foo', '-5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', 'foo', 'asdf'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', 'foo', '5ds'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', 'foo', '-5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', 'foo', 'asdf'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', 'foo', '5ds'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', 'foo', '-5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', 'foo', 'asdf'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', 'foo', '5ds'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', 'foo', '-5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', 'foo', 'asdf'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', 'foo', '5ds'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', 'foo', '-5'))

    def test_checksum_short(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', 'foo', '0', 'md5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list,
                ('MANIFEST', 'foo', '0', 'md5', 'd41d8cd98f00b204e9800998ecf8427e', 'sha1'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', 'foo', '0', 'md5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ('DATA', 'foo', '0', 'md5', 'd41d8cd98f00b204e9800998ecf8427e', 'sha1'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', 'foo', '0', 'md5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list,
                ('MISC', 'foo', '0', 'md5', 'd41d8cd98f00b204e9800998ecf8427e', 'sha1'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', 'foo', '0', 'md5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list,
                ('DIST', 'foo', '0', 'md5', 'd41d8cd98f00b204e9800998ecf8427e', 'sha1'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', 'foo', '0', 'md5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list,
                ('EBUILD', 'foo', '0', 'md5', 'd41d8cd98f00b204e9800998ecf8427e', 'sha1'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', 'foo', '0', 'md5'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list,
                ('AUX', 'foo', '0', 'md5', 'd41d8cd98f00b204e9800998ecf8427e', 'sha1'))

    def test_invalid_value_count(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMANIFEST.from_list, ('MANIFEST', 'foo'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryIGNORE.from_list, ('IGNORE',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryIGNORE.from_list, ('IGNORE', 'foo', 'bar'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list, ('DATA', 'foo'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryMISC.from_list, ('MISC', 'foo'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDIST.from_list, ('DIST', 'foo'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryEBUILD.from_list, ('EBUILD', 'foo'))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX',))
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryAUX.from_list, ('AUX', 'foo'))


class ManifestUtilityTest(unittest.TestCase):
    def test_manifest_hashes_to_hashlib(self):
        self.assertListEqual(list(
                gemato.manifest.manifest_hashes_to_hashlib(['MD5', 'SHA1'])),
            ['md5', 'sha1'])
        self.assertListEqual(list(
                gemato.manifest.manifest_hashes_to_hashlib(['RMD160'])),
            ['ripemd160'])
        self.assertListEqual(list(
                gemato.manifest.manifest_hashes_to_hashlib(['SHA3_256', 'SHA256'])),
            ['sha3_256', 'sha256'])


class NewManifestEntryTest(unittest.TestCase):
    """
    Tests for new_manifest_entry().
    """

    def test_TIMESTAMP(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('TIMESTAMP',
                    datetime.datetime(2000, 1, 1, 0, 0, 0)),
                gemato.manifest.ManifestEntryTIMESTAMP)

    def test_MANIFEST(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('MANIFEST',
                    'test', 32, {}),
                gemato.manifest.ManifestEntryMANIFEST)

    def test_IGNORE(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('IGNORE', 'test'),
                gemato.manifest.ManifestEntryIGNORE)

    def test_DATA(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('DATA',
                    'test', 32, {}),
                gemato.manifest.ManifestEntryDATA)

    def test_MISC(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('MISC',
                    'test', 32, {}),
                gemato.manifest.ManifestEntryMISC)

    def test_DIST(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('DIST',
                    'test', 32, {}),
                gemato.manifest.ManifestEntryDIST)

    def test_EBUILD(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('EBUILD',
                    'test', 32, {}),
                gemato.manifest.ManifestEntryEBUILD)

    def test_AUX(self):
        self.assertIsInstance(
                gemato.manifest.new_manifest_entry('AUX',
                    'test', 32, {}),
                gemato.manifest.ManifestEntryAUX)


class ManifestPathEncodingTest(unittest.TestCase):
    """
    Tests for path encoding.
    """

    def test_encode_space_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            'tes t', 32, {})
        self.assertEqual(m.path, 'tes t')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\x20t', '32'])

    def test_encode_tab_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            'tes\tt', 32, {})
        self.assertEqual(m.path, 'tes\tt')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\x09t', '32'])

    def test_encode_nbsp_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            u'tes\u00a0t', 32, {})
        self.assertEqual(m.path, u'tes\u00a0t')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\u00A0t', '32'])

    def test_encode_en_quad_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            u'tes\u2000t', 32, {})
        self.assertEqual(m.path, u'tes\u2000t')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\u2000t', '32'])

    def test_encode_null_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            'tes\x00t', 32, {})
        self.assertEqual(m.path, 'tes\x00t')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\x00t', '32'])

    def test_encode_bell_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            'tes\at', 32, {})
        self.assertEqual(m.path, 'tes\at')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\x07t', '32'])

    def test_encode_del_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            'tes\x7Ft', 32, {})
        self.assertEqual(m.path, 'tes\x7Ft')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\x7Ft', '32'])

    def test_encode_pad_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            u'tes\u0080t', 32, {})
        self.assertEqual(m.path, u'tes\u0080t')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\u0080t', '32'])

    def test_encode_backslash_in_filename(self):
        m = gemato.manifest.new_manifest_entry('DATA',
            'tes\\t', 32, {})
        self.assertEqual(m.path, 'tes\\t')
        self.assertListEqual(list(m.to_list()),
                ['DATA', 'tes\\x5Ct', '32'])

    def test_decode_space_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x20t', 32])
        self.assertEqual(m.path, 'tes t')

    def test_decode_space_in_filename_u(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\u0020t', 32])
        self.assertEqual(m.path, 'tes t')

    def test_decode_space_in_filename_lu(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\U00000020t', 32])
        self.assertEqual(m.path, 'tes t')

    def test_decode_tab_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x09t', 32])
        self.assertEqual(m.path, 'tes\tt')

    def test_decode_nbsp_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\u00A0t', 32])
        self.assertEqual(m.path, u'tes\u00a0t')

    def test_decode_nbsp_in_filename_lc(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\u00a0t', 32])
        self.assertEqual(m.path, u'tes\u00a0t')

    def test_decode_en_quad_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\u2000t', 32])
        self.assertEqual(m.path, u'tes\u2000t')

    def test_decode_null_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x00t', 32])
        self.assertEqual(m.path, 'tes\x00t')

    def test_decode_bell_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x07t', 32])
        self.assertEqual(m.path, 'tes\at')

    def test_decode_del_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x7Ft', 32])
        self.assertEqual(m.path, 'tes\x7Ft')

    def test_decode_del_in_filename_lc(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x7ft', 32])
        self.assertEqual(m.path, 'tes\x7Ft')

    def test_decode_pad_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\u0080t', 32])
        self.assertEqual(m.path, u'tes\u0080t')

    def test_decode_backslash_in_filename(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x5Ct', 32])
        self.assertEqual(m.path, 'tes\\t')

    def test_decode_backslash_in_filename_lc(self):
        m = gemato.manifest.ManifestEntryDATA.from_list(['DATA',
            'tes\\x5ct', 32])
        self.assertEqual(m.path, 'tes\\t')

    def test_decode_invalid_backslash_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\t', 32])

    def test_decode_double_backslash_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\\\t', 32])

    def test_decode_trailing_backslash_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\', 32])

    def test_decode_empty_x_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\xt', 32])

    def test_decode_short_x_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\x5t', 32])

    def test_decode_empty_u_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\ut', 32])

    def test_decode_short_u_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\u345t', 32])

    def test_decode_empty_lu_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\Ut', 32])

    def test_decode_short_lu_in_filename(self):
        self.assertRaises(gemato.exceptions.ManifestSyntaxError,
                gemato.manifest.ManifestEntryDATA.from_list,
                ['DATA', 'tes\\U0000345t', 32])



# gemato: Manifest file objects
# vim:fileencoding=utf-8
# (c) 2017-2018 Michał Górny
# Licensed under the terms of 2-clause BSD license

import datetime
import io
import os.path
import re
import sys

import gemato.exceptions
import gemato.openpgp
import gemato.util


class ManifestEntryTIMESTAMP(object):
    """
    ISO-8601 timestamp.
    """

    __slots__ = ['ts']
    tag = 'TIMESTAMP'

    def __init__(self, ts):
        assert isinstance(ts, datetime.datetime)
        self.ts = ts

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        if len(l) != 2:
            raise gemato.exceptions.ManifestSyntaxError(
                    '{} line: expects 1 value, got: {}'.format(l[0], l[1:]))
        try:
            ts = datetime.datetime.strptime(l[1], '%Y-%m-%dT%H:%M:%SZ')
        except ValueError:
            raise gemato.exceptions.ManifestSyntaxError(
                    '{} line: expected ISO8601 timestamp, got: {}'.format(l[0], l[1:]))
        return cls(ts)

    def to_list(self):
        return (self.tag, self.ts.strftime('%Y-%m-%dT%H:%M:%SZ'))

    def __eq__(self, other):
        return self.tag == other.tag and self.ts == other.ts

    def __lt__(self, other):
        return (self.tag < other.tag
                or (self.tag == other.tag and self.ts < other.ts))


if sys.hexversion >= 0x03000000:
    unichr = chr


class ManifestPathEntry(object):
    """
    Base class for entries using a path.
    """

    __slots__ = ['path']
    disallowed_path_re = re.compile(r'[\x00-\x1F\x7F-\x9F\s\\]', re.U)
    escape_seq_re = re.compile(r'\\(x[0-9a-fA-F]{2}|u[0-9a-fA-F]{4}|U[0-9a-fA-F]{8})?')

    def __init__(self, path):
        assert path[0] != '/'
        self.path = path

    @staticmethod
    def decode_char(m):
        val = m.group(1)
        if val is None:
            raise gemato.exceptions.ManifestSyntaxError(
                    'Invalid escape sequence at pos {} of: {}'.format(m.start(), m.string))
        return unichr(int(val[1:], base=16))

    @classmethod
    def process_path(cls, l):
        if len(l) != 2:
            raise gemato.exceptions.ManifestSyntaxError(
                    '{} line: expects 1 value, got: {}'.format(l[0], l[1:]))
        if not l[1] or l[1][0] == '/':
            raise gemato.exceptions.ManifestSyntaxError(
                    '{} line: expected relative path, got: {}'.format(l[0], l[1:]))
        return cls.escape_seq_re.sub(cls.decode_char, l[1])

    @staticmethod
    def encode_char(m):
        assert len(m.group(0)) == 1
        cp = ord(m.group(0))
        if cp <= 0x7F:
            return '\\x{:02X}'.format(cp)
        elif cp <= 0xFFFF:
            return '\\u{:04X}'.format(cp)
        else:
            return '\\U{:08X}'.format(cp)

    @property
    def encoded_path(self):
        return self.disallowed_path_re.sub(self.encode_char, self.path)

    def __eq__(self, other):
        return self.tag == other.tag and self.path == other.path

    def __lt__(self, other):
        return (self.tag < other.tag
                or (self.tag == other.tag and self.path < other.path))


class ManifestEntryIGNORE(ManifestPathEntry):
    """
    Ignored path.
    """

    tag = 'IGNORE'

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        return cls(cls.process_path(l))

    def to_list(self):
        return (self.tag, self.encoded_path)


class ManifestFileEntry(ManifestPathEntry):
    """
    Base class for entries providing checksums for a path.
    """

    __slots__ = ['checksums', 'size']

    def __init__(self, path, size, checksums):
        super(ManifestFileEntry, self).__init__(path)
        self.size = size
        self.checksums = checksums

    @staticmethod
    def process_checksums(l):
        if len(l) < 3:
            raise gemato.exceptions.ManifestSyntaxError(
                    '{} line: expects at least 2 values, got: {}'.format(l[0], l[1:]))

        try:
            size = int(l[2])
            if size < 0:
                raise ValueError()
        except ValueError:
            raise gemato.exceptions.ManifestSyntaxError(
                    '{} line: size must be a non-negative integer, got: {}'.format(l[0], l[2]))

        checksums = {}
        it = iter(l[3:])
        while True:
            try:
                ckname = next(it)
            except StopIteration:
                break
            try:
                ckval = next(it)
            except StopIteration:
                raise gemato.exceptions.ManifestSyntaxError(
                        '{} line: checksum {} has no value'.format(l[0], ckname))
            checksums[ckname] = ckval

        return size, checksums

    def to_list(self, tag):
        ret = [tag, self.encoded_path, str(self.size)]
        for k, v in sorted(self.checksums.items()):
            ret += [k, v]
        return ret

    def __eq__(self, other):
        return (super(ManifestFileEntry, self).__eq__(other)
                and self.size == other.size
                and self.checksums == other.checksums)

    # for the purpose of __lt__, the path is good enough for sorting


class ManifestEntryMANIFEST(ManifestFileEntry):
    """
    Sub-Manifest file reference.
    """

    tag = 'MANIFEST'

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        path = cls.process_path(l[:2])
        size, checksums = cls.process_checksums(l)
        return cls(path, size, checksums)

    def to_list(self):
        return super(ManifestEntryMANIFEST, self).to_list(self.tag)


class ManifestEntryDATA(ManifestFileEntry):
    """
    Regular file reference.
    """

    tag = 'DATA'

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        path = cls.process_path(l[:2])
        size, checksums = cls.process_checksums(l)
        return cls(path, size, checksums)

    def to_list(self):
        return super(ManifestEntryDATA, self).to_list(self.tag)


class ManifestEntryDIST(ManifestFileEntry):
    """
    Distfile reference.
    """

    tag = 'DIST'

    @classmethod
    def from_list(cls, l):
        path = cls.process_path(l[:2])
        if '/' in path:
            raise gemato.exceptions.ManifestSyntaxError(
                    'DIST line: file name expected, got directory path: {}'.format(path))
        size, checksums = cls.process_checksums(l)
        return cls(path, size, checksums)

    def to_list(self):
        return super(ManifestEntryDIST, self).to_list(self.tag)


class ManifestEntryEBUILD(ManifestFileEntry):
    """
    Deprecated ebuild file reference (equivalent to DATA).
    """

    tag = 'EBUILD'

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        path = cls.process_path(l[:2])
        size, checksums = cls.process_checksums(l)
        return cls(path, size, checksums)

    def to_list(self):
        return super(ManifestEntryEBUILD, self).to_list(self.tag)


class ManifestEntryMISC(ManifestFileEntry):
    """
    Deprecated 'non-strict' checksum (now equivalent to DATA).
    """

    tag = 'MISC'

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        path = cls.process_path(l[:2])
        size, checksums = cls.process_checksums(l)
        return cls(path, size, checksums)

    def to_list(self):
        return super(ManifestEntryMISC, self).to_list(self.tag)


class ManifestEntryAUX(ManifestFileEntry):
    """
    Deprecated AUX file reference (DATA with 'files/' prepended).
    """

    __slots__ = ['aux_path']
    tag = 'AUX'

    def __init__(self, aux_path, size, checksums):
        self.aux_path = aux_path
        super(ManifestEntryAUX, self).__init__(
                os.path.join('files', aux_path), size, checksums)

    @classmethod
    def from_list(cls, l):
        assert l[0] == cls.tag
        path = cls.process_path(l[:2])
        size, checksums = cls.process_checksums(l)
        return cls(path, size, checksums)

    def to_list(self):
        ret = super(ManifestEntryAUX, self).to_list(self.tag)
        assert gemato.util.path_inside_dir(ret[1], 'files')
        ret[1] = ret[1][6:]
        return ret


MANIFEST_TAG_MAPPING = {
    'TIMESTAMP': ManifestEntryTIMESTAMP,
    'MANIFEST': ManifestEntryMANIFEST,
    'IGNORE': ManifestEntryIGNORE,
    'DATA': ManifestEntryDATA,
    'DIST': ManifestEntryDIST,
    'EBUILD': ManifestEntryEBUILD,
    'MISC': ManifestEntryMISC,
    'AUX': ManifestEntryAUX,
}


def new_manifest_entry(tag, *args):
    """
    Construct a Manifest entry for given @tag. @args are passed
    to the constructor.
    """
    return MANIFEST_TAG_MAPPING[tag](*args)


class ManifestState(object):
    """
    FSM constants for loading Manifest.
    """

    DATA = 0
    SIGNED_PREAMBLE = 1
    SIGNED_DATA = 2
    SIGNATURE = 3
    POST_SIGNED_DATA = 4


class ManifestFile(object):
    """
    A class encapsulating a single Manifest file. It supports reading
    from files and writing to them.
    """

    __slots__ = ['entries', 'openpgp_signed', 'openpgp_signature']

    def __init__(self, f=None):
        """
        Create a new instance. If @f is provided, reads the entries
        from open Manifest file @f (see load()).
        """

        self.entries = []
        self.openpgp_signed = None
        self.openpgp_signature = None
        if f is not None:
            self.load(f)

    def load(self, f, verify_openpgp=True, openpgp_env=None):
        """
        Load data from file @f. The file should be open for reading
        in text mode, and oriented at the beginning.

        If @verify_openpgp is True and the Manifest contains an OpenPGP
        signature, the signature will be verified. In that case,
        @openpgp_env needs to be provided.

        If the verification succeeds, the openpgp_signed property will
        be set to True and openpgp_signature will contain the signature
        data. If it fails or OpenPGP is not available, an exception
        will be raised. If the exception is caught, the caller
        can continue using the ManifestFile instance -- it will
        be loaded completely.
        """

        self.entries = []
        self.openpgp_signed = False
        self.openpgp_signature = None
        state = ManifestState.DATA
        openpgp_data = ''

        for l in f:
            if state == ManifestState.DATA:
                if l == '-----BEGIN PGP SIGNED MESSAGE-----\n':
                    if self.entries:
                        raise gemato.exceptions.ManifestUnsignedData()
                    if verify_openpgp:
                        openpgp_data += l
                    state = ManifestState.SIGNED_PREAMBLE
                    continue
            elif state == ManifestState.SIGNED_PREAMBLE:
                if verify_openpgp:
                    openpgp_data += l
                # skip header lines up to the empty line
                if l.strip():
                    continue
                state = ManifestState.SIGNED_DATA
            elif state == ManifestState.SIGNED_DATA:
                if verify_openpgp:
                    openpgp_data += l
                if l == '-----BEGIN PGP SIGNATURE-----\n':
                    state = ManifestState.SIGNATURE
                    continue
                # dash-escaping, RFC 4880 says any line can suffer from it
                if l.startswith('- '):
                    l = l[2:]
            elif state == ManifestState.SIGNATURE:
                if verify_openpgp:
                    openpgp_data += l
                if l == '-----END PGP SIGNATURE-----\n':
                    state = ManifestState.POST_SIGNED_DATA
                    continue

            if l.startswith('-----') and l.rstrip().endswith('-----'):
                raise gemato.exceptions.ManifestSyntaxError(
                        "Unexpected OpenPGP header: {}".format(l))
            if state in (ManifestState.SIGNED_PREAMBLE, ManifestState.SIGNATURE):
                continue

            sl = l.strip().split()
            # skip empty lines
            if not sl:
                continue
            if state == ManifestState.POST_SIGNED_DATA:
                raise gemato.exceptions.ManifestUnsignedData()
            tag = sl[0]
            try:
                self.entries.append(MANIFEST_TAG_MAPPING[tag]
                        .from_list(sl))
            except KeyError:
                raise gemato.exceptions.ManifestSyntaxError(
                        "Invalid Manifest line: {}".format(l))

        if state == ManifestState.SIGNED_PREAMBLE:
            raise gemato.exceptions.ManifestSyntaxError(
                    "Manifest terminated early, in OpenPGP headers")
        elif state == ManifestState.SIGNED_DATA:
            raise gemato.exceptions.ManifestSyntaxError(
                    "Manifest terminated early, before signature")
        elif state == ManifestState.SIGNATURE:
            raise gemato.exceptions.ManifestSyntaxError(
                    "Manifest terminated early, inside signature")

        if verify_openpgp and state == ManifestState.POST_SIGNED_DATA:
            assert openpgp_env
            with io.StringIO(openpgp_data) as f:
                self.openpgp_signature = openpgp_env.verify_file(f)
            self.openpgp_signed = True

    def dump(self, f, sign_openpgp=None, openpgp_keyid=None,
            openpgp_env=None, sort=False):
        """
        Dump data into file @f. The file should be open for writing
        in text mode, and truncated to zero length.

        If @sign_openpgp is True, the file will include an OpenPGP
        cleartext signature. If it False, the signature will be omitted.
        If it is None (the default), the file will be signed if it
        was originally signed with a valid signature.

        If @openpgp_keyid is not None, it specifies the key to use
        for signing. If @sign_openpgp is not False, @openpgp_env must
        provide an OpenPGP environment for signing.

        If @sort is True, the entries are sorted prior to dumping.
        """

        if sign_openpgp is None:
            sign_openpgp = self.openpgp_signed

        if sort:
            self.entries = sorted(self.entries)

        if sign_openpgp:
            assert openpgp_env
            with io.StringIO() as data:
                # get the plain data into a stream
                self.dump(data, sign_openpgp=False)
                data.seek(0)
                openpgp_env.clear_sign_file(data, f, keyid=openpgp_keyid)
        else:
            for e in self.entries:
                f.write(u' '.join(e.to_list()) + '\n')

    def find_timestamp(self):
        """
        Find a timestamp entry and return it. Returns None if there
        is no timestamp.
        """

        for e in self.entries:
            if e.tag == 'TIMESTAMP':
                return e
        return None

    def find_path_entry(self, path):
        """
        Find a matching entry for path @path and return it. Returns
        None when no path matches. DIST entries are not included.
        """

        for e in self.entries:
            if e.tag == 'IGNORE':
                # ignore matches recursively, so we process it separately
                # py<3.5 does not have os.path.commonpath()
                if gemato.util.path_starts_with(path, e.path):
                    return e
            elif e.tag in ('DIST', 'TIMESTAMP'):
                # distfiles are not local files, so skip them
                # timestamp is not a file ;-)
                pass
            else:
                if e.path == path:
                    return e
        return None

    def find_dist_entry(self, filename):
        """
        Find a matching entry for distfile @filename and return it.
        Returns None when no DIST entry matches.
        """

        for e in self.entries:
            if e.tag == 'DIST' and e.path == filename:
                return e
        return None

    def find_manifests_for_path(self, path):
        """
        Find all MANIFEST entries that could affect the path @path
        and return an iterator over them. Yield an empty list when
        there are no matching MANIFEST entries.
        """

        for e in self.entries:
            if e.tag == 'MANIFEST':
                mdir = os.path.dirname(e.path)
                if gemato.util.path_inside_dir(path, mdir):
                    yield e


MANIFEST_HASH_MAPPING = {
    'MD5': 'md5',
    'SHA1': 'sha1',
    'SHA256': 'sha256',
    'SHA512': 'sha512',
    'RMD160': 'ripemd160',
    'WHIRLPOOL': 'whirlpool',
    'BLAKE2B': 'blake2b',
    'BLAKE2S': 'blake2s',
    'SHA3_256': 'sha3_256',
    'SHA3_512': 'sha3_512',
}


def manifest_hashes_to_hashlib(hashes):
    """
    Return the hashlib hash names corresponding to the Manifest names
    in @hashes. Returns an iterable.
    """
    for h in hashes:
        yield MANIFEST_HASH_MAPPING[h]



#!/usr/bin/python
# vim:fileencoding=utf-8
# (C) 2017-2018 Michał Górny <mgorny@gentoo.org>
# Licensed under the terms of 2-clause BSD license

from setuptools import setup


setup(
    name='gemato',
    version='10.3',
    description='Gentoo Manifest Tool -- a stand-alone utility to verify and update Gentoo Manifest files',

    author='Michał Górny',
    author_email='mgorny@gentoo.org',
    license='BSD',
    url='http://github.com/mgorny/gemato',

    extras_require={
        'blake2': ['pyblake2;python_version<"3.6"'],
        'bz2': ['bz2file;python_version<"3.0"'],
        'lzma': ['backports.lzma;python_version<"3.0"'],
        'sha3': ['pysha3;python_version<"3.6"'],
    },

    packages=['gemato'],
    entry_points={
        'console_scripts': [
            'gemato=gemato.cli:setuptools_main',
        ],
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Security :: Cryptography',
    ]
)
